function filter_compare(N,Rp,Rs,w,Nbits)
%function filter_compare(N,Rp,Rs,w,Nbits)
%
% compare 2nd order section filtering with direct form filter
% (using elliptic filter)
%
% N     filter order
% Rp    pass band ripple in dB
% Rs    stop band attenuation in dB
% w     cutoff frequency (low pass), [w1 w2} (band pass)
%       normalized to Fs/2
% Nbits coefficient word length
%
% example: filter_compare(5,0.5,50,1/8,12); 
%
% G. Doblinger, 12-2000, TU-Wien

if nargin == 0
   help filter_compare
   return
elseif nargin == 4
   Nbits = inf;
elseif nargin < 4
   error('wrong number of input arguments');
end

% design elliptic filter, compute cascade form

x = [1 zeros(1,2047)];
[z0,p0,k] = ellip(N,Rp,Rs,w);
C = zp2sos(z0,p0,k);            % compute coeff. of 2nd order sections
[Nr,Nc] = size(C);
y = filter(C(1,1:3),C(1,4:6),x);            % first section
for n = 2:Nr
   y = filter(C(n,1:3),C(n,4:6),y);         % remaining sections
end 
if Nbits ~= inf
   scale = 2^(Nbits-1);
   Cq = 1/scale*round(scale*C);
   yq = filter(Cq(1,1:3),C(1,4:6),x);       % first section (quantized)
   for n = 2:Nr
      yq = filter(Cq(n,1:3),Cq(n,4:6),yq);  % remaining sections
   end
   errc = 100*norm(y-yq)/norm(y);
   disp(['Cascade form impulse response RMS error = ', num2str(errc),'%']);
end 

% repeat for parallel form

[b,a] = ellip(N,Rp,Rs,w);
[r,p0,k] = residuez(b,a);
r = r(:).';
p0 = p0(:).';
if length(r) ~= N
   disp('INFO: multiple poles not supported with parallel form');
   pflag = 0;
else
   pflag = 1;
end

if pflag
   
   % find real poles, compute first order section coeff.

   tiny = 1e-11;
   ipr = find(abs(imag(p0)) < tiny);
   Nipr = length(ipr);
   if Nipr
      C = [real(r(ipr)) ; zeros(1,Nipr) ; zeros(1,Nipr) ; ones(1,Nipr) ; ... 
	   -real(p0(ipr)) ; zeros(1,Nipr)];
   else
      C = [];
   end   

   % find conjugate complex poles (pos. imag. part only), compute biquad coeff.

   ipc = find(imag(p0) >= tiny);
   Nipc = length(ipc);
   if ~isempty(ipc)
      C = [C , [2*real(r(ipc)) ; -2*real(r(ipc).*conj(p0(ipc))) ; zeros(1,Nipc) ; ... 
		ones(1,Nipc) ; -2*real(p0(ipc)) ; abs(p0(ipc)).^2]];
   end
   C = C.';
   [Nr,Nc] = size(C);
   yp = filter(k,1,x);
   for n = 1:Nr
      yp = yp + filter(C(n,1:3),C(n,4:6),x);
   end 
   if Nbits ~= inf
      Cq = 1/scale*round(scale*C);
      kq = 1/scale*round(scale*k);
      ypq = filter(kq,1,x);
      for n = 1:Nr
	 ypq = ypq + filter(Cq(n,1:3),Cq(n,4:6),x);
      end 
      errp = 100*norm(yp-ypq)/norm(yp);
      disp(['Parallel form impulse response RMS error = ', num2str(errp),'%']);
   end
end

% repeat for direct form

yd = filter(b,a,x);
if Nbits ~= inf
   bq = 1/scale*round(scale*b);
   aq = 1/scale*round(scale*a);
   ydq = filter(bq,aq,x);
   errd = 100*norm(yd-ydq)/norm(yd);
   disp(['Direct form impulse response signal RMS error = ', num2str(errd),'%']);
end

% plot DFT magnitudes

if Nbits == inf
   disp('No difference for floating point precision');
   return
end
if errc >= 100
   disp('*** Cascade form shows extreme error ***');
end
if errd > 100
   disp('*** Direct form shows extreme error ***');
end
if errp > 100
   disp('*** Parallel form shows extreme error ***');
end

close all
if strcmp(upper(computer),'PCWIN')
   pos1 = [0.5 0.4 0.49 0.5];
   axfnt = 9;
   ticfnt = 8;
   yoff = 0.49;
else	
   pos1 = [0.5 0.6 0.49 0.4];
   axfnt = 13;
   ticfnt = 12;
   yoff = 0.6;
end

fmag = figure('numbertitle','off','name',...
   'Cascade form: DFT magnitude (y-zoom an)',...
   'Units','normal','Position',pos1);
colordef(fmag,'black');

Nf = max(2048,length(y));
Y = fft(y,Nf);
Nfh = round(Nf/2)+1;
Ymag = 20*log10(abs(Y(1:Nfh))+eps);
Yq = fft(yq,Nf);
Yqmag = 20*log10(abs(Yq(1:Nfh))+eps);
f = linspace(0,1,Nfh);
plot(f,Ymag,'g',f,Yqmag,'r');
xlabel('Frequency normalized to Fs/2');
ylabel('Magnitude in dB');
title(['Cascade form, Nbits = ',int2str(Nbits)]);
grid on
legend('exact','quantized');
axis([-inf inf -2*Rs 20]);
zoom(fmag,'yon');


pos1(1) = pos1(1) - 0.495;  % position of next figure
fmagp = figure('numbertitle','off','name',...
   'Parallel form: DFT magnitude (y-zoom an)',...
   'Units','normal','Position',pos1);
colordef(fmagp,'black');

Y = fft(yp,Nf);
Ymag = 20*log10(abs(Y(1:Nfh))+eps);
Yq = fft(ypq,Nf);
Yqmag = 20*log10(abs(Yq(1:Nfh))+eps);
plot(f,Ymag,'g',f,Yqmag,'r');
xlabel('Frequency normalized to Fs/2');
ylabel('Magnitude in dB');
title(['Parallel form, Nbits = ',int2str(Nbits)]);
grid on
legend('exact','quantized');
axis([-inf inf -2*Rs 20]);
zoom(fmagp,'yon');

pos1(2) = pos1(2) - yoff;
fmagd = figure('numbertitle','off','name',...
   'Direct form: DFT magnitude (y-zoom an)',...
   'Units','normal','Position',pos1);
colordef(fmagd,'black');

Y = fft(yd,Nf);
Ymag = 20*log10(abs(Y(1:Nfh))+eps);
Yq = fft(ydq,Nf);
Yqmag = 20*log10(abs(Yq(1:Nfh))+eps);
plot(f,Ymag,'g',f,Yqmag,'r');
xlabel('Frequency normalized to Fs/2');
ylabel('Magnitude in dB');
title(['Direct form, Nbits = ',int2str(Nbits)]);
grid on
legend('exact','quantized');
axis([-inf inf -2*Rs 20]);
zoom(fmagd,'yon');
