function [y,s] = state_filt(b,a,x,s)
%function [y,s] = state_filt(b,a,x,s)
%
% discrete-time state-space filtering for scalar inputs
% 
% a,b   numerator and denominator of transfer function
% x     input signal vector 
% s     state vector, length = max(length(a),length(b))-1
%       (initial state set to zero, if ommited)
% y     output signal vector
%
% G. Doblinger, 12-2000, TU-Wien

if nargin == 0
   help state_filt
   return
end
Na = length(a);
Nb = length(b);
Ns = max(Na,Nb)-1;
if nargin == 3
   s = zeros(Ns,1);
elseif length(s) ~= Ns
   error('inproper length of state vector');
end

[A,B,C,D] = tf2ss(b,a);
y = zeros(size(x));
snorm = 0;

% apply L2 scaling to limit state vector norm

delta = 4;    % assumed crest factor of input signal
K = Kmatrix(a);
scale = delta*sqrt(K(1,1));
T = scale*eye(Ns);
Ti = 1/scale*eye(Ns);
%A = Ti*A*T   % not needed, since A is not changed here
B = Ti*B;
C = C*T;

for n = 1:length(x)
   y(n) = C*s + D*x(n); 
   s = A*s;
   s(1) = s(1) + B(1)*x(n);
   snorm = max(norm(s),snorm);
end

disp(sprintf('Maximum state vector norm = %f', snorm));

function K = Kmatrix(a,Nfft)
%function K = Kmatrix(a,Nfft)
%
% compute K-matrix for state-space analysis
% K = AKA' + BB'
% with state-space matrices A,B, B = (1,0,0...)'
%
% a      denominator poly. of transfer function
% Nfft   number of FFT points used
%        Nfft = 512, if omitted
%
% G. Doblinger, 12-2000, TU-Wien

if nargin == 0
   help Kmatrix
   return
elseif nargin == 1
   Nfft = 512;
else
   Nfft = 2^(ceil(log2(Nfft)));
end

N = length(a);
a = a(:).';

% compute correlation matrix of state vector
% (white gaussian noise at input)

Sa = fft(a,Nfft);
r = ifft(1./(abs(Sa).^2));
R = toeplitz(real(r(1:N)));

% extract K-matrix from R
% (see Roberts, Mullis book page 519)

K = R(2:N,2:N);

% check error

A = [-a(2:N) ; [eye(N-2) zeros(N-2,1)]];
E = K - A*K*A';
disp(sprintf('Error norm = %2.10e', abs(norm(E)-1)));
