function [z,p,k] = bwdigfi(n,rp,Wn,ftyp)
% function [z,p,k] = bwdigfi(n,rp,Wn,ftyp) or [z,p,k] = bwdigfi(n,rp,Wn)
%
% Digital Butterworth filter using bilinear transform, and digital
% frequency transforms (for HP, BP, BS)
%
% n      filter order of LP, HP (BP and BS will have order 2n)
% rp     attenuation in dB at cut-off frequency
% Wn     cutoff frequency between 0 and 1 (1 = half the sampling rate)
%        if Wn = [w1 w2], then a BP with passband w1 < w < w2 is designed
% ftyp   'high' for HP, 'stop' for BS, omitted otherwise
% z      vector of zeros
% p      vector of poles
% k      gain factor of transfer function
%
% G. Doblinger, TU-Wien, 8-94

if nargin == 0
   help bwdigfi
   return
elseif nargin == 3
   if length(Wn) < 2
      filt = 1;         % low pass filter
   else   
      filt = 2;         % band pass filter
   end   
elseif nargin == 4
   if ftyp == 'high' | ftyp == 'HIGH'
      filt = 3;
   elseif ftyp == 'stop' | ftyp == 'STOP'
      filt = 4;
   else
      error('last argument must be a string');
   end
else
   error('Wrong number of arguments');
end                  

% determine analog prototype low pass filter (with 3dB cut-off freq.)
   
[za,pa,ka] = buttap1(n,rp);
   
% apply bilinear transform, and frequency transform afterwards (if
% required), special (simplified) frequency transforms are used
% (Constantinides-Transforms with k = 1 (and alpha = 0 for HP only))
   
if filt == 1         % LP-filter
   [z,p,k] = bilzt(za, pa, ka, Wn);
elseif filt == 2     % BP-filter
   w1 = Wn(1);
   w2 = Wn(2);
   [z,p,k] = bilzt(za, pa, ka, abs(w2 - w1));   
   z = ones(n, 1);
   z = [z ; -z];     % multiple zeros at +-1
   alpha = cos(pi * (w2 + w1) / 2) / cos(pi * (w2 - w1) / 2);
   x1 = 0.5 * alpha * (1 + p);
   x2 = sqrt(x1.^2 - p);
   p = [(x1 + x2) ; (x1 - x2)];
elseif filt == 3     % HP-filter
   [z,p,k] = bilzt(za, pa, ka, (1 - Wn));
   z = -z;
   p = -p;       
elseif filt == 4     % BS-filter
   w1 = Wn(1);
   w2 = Wn(2);
   [z,p,k] = bilzt(za, pa, ka, (1 - abs(w2 - w1)));   
   alpha = cos(pi * (w2 + w1) / 2) / cos(pi * (w2 - w1) / 2);
   ta = sqrt(alpha^2 - 1);
   z = ones(n, 1);
   z = [((alpha + ta) * z) ; ((alpha - ta) * z)];
   x1 = 0.5 * alpha * (1 - p);
   x2 = sqrt(x1.^2 + p);
   p = [(x1 + x2) ; (x1 - x2)];   
end      
   
