function [z,p,k] = c2digfi(n,Rs,Ws,ftyp)
% function [z,p,k] = c2digfi(n,Rs,Ws,ftyp) or [z,p,k] = c2digfi(n,Rs,Ws)
%
% Digital Chebyshev type II filter using bilinear transform, and digital
% frequency transforms (for HP, BP, BS)
%
% n      filter order of LP, HP (BP and BS will have order 2n)
% Rs     stop band ripple in dB
% Ws     cutoff frequency (of stop band!) between 0 and 1 (1 = half the sampling rate)
%        if Ws = [w1 w2], then a BP with passband w1 < w < w2 is designed
% ftyp   'high' for HP, 'stop' for BS, omitted otherwise
% z      vector of zeros
% p      vector of poles
% k      gain factor of transfer function
%
% G. Doblinger, TU-Wien, 8-1994

if nargin == 0
   help c2digfi
   return
elseif nargin == 3
   if length(Ws) < 2
      filt = 1;         % low pass filter
   else   
      filt = 2;         % band pass filter
   end   
elseif nargin == 4
   if ftyp == 'high' | ftyp == 'HIGH'
      filt = 3;
   elseif ftyp == 'stop' | ftyp == 'STOP'
      filt = 4;
   else
      error('last argument must be a string');
   end
else
   error('Wrong number of arguments');
end                  

% determine analog prototype low pass filter
   
[za,pa,ka] = cheb2ap(n, Rs);
   
% apply bilinear transform, and frequency transform afterwards (if
% required), special (simplified) frequency transforms are used
% (Constantinides-Transforms with k = 1 (and alpha = 0 for HP only))
   
if filt == 1         % LP-filter
   [z,p,k] = bilzt(za, pa, ka, Ws);
elseif filt == 2     % BP-filter
   w1 = Ws(1);
   w2 = Ws(2);
   [z,p,k] = bilzt(za, pa, ka, abs(w2 - w1));   
   alpha = cos(pi * (w2 + w1) / 2) / cos(pi * (w2 - w1) / 2);
   x1 = 0.5 * alpha * (1 + z);
   x2 = sqrt(x1.^2 - z);
   z = [(x1 + x2) ; (x1 - x2)];
   x1 = 0.5 * alpha * (1 + p);
   x2 = sqrt(x1.^2 - p);
   p = [(x1 + x2) ; (x1 - x2)];
elseif filt == 3     % HP-filter
   [z,p,k] = bilzt(za, pa, ka, 1 - Ws);
   z = -z;
   p = -p;       
elseif filt == 4     % BS-filter
   w1 = Ws(1);
   w2 = Ws(2);
   [z,p,k] = bilzt(za, pa, ka, 1 - abs(w2 - w1));   
   alpha = cos(pi * (w2 + w1) / 2) / cos(pi * (w2 - w1) / 2);
   x1 = 0.5 * alpha * (1 - z);
   x2 = sqrt(x1.^2 + z);
   z = [(x1 + x2) ; (x1 - x2)];
   x1 = 0.5 * alpha * (1 - p);
   x2 = sqrt(x1.^2 + p);
   p = [(x1 + x2) ; (x1 - x2)];
end      
   
