function h = lawson(N,om,D,W,iter)
% h = lawson(N,om,D,W,iter)
% Complex Chebyshev FIR filter design using Lawson's algorithm
%
% h      filter impulse response
% N      filter length
% om     frequency grid (0 <= om <= pi)
% D      complex desired frequency response on the grid om
% W      positive weighting function on the grid om
% iter   maximum number of iterations
%
% example: length 61 bandpass, band edges [.23,.3,.5,.57]*pi,
% weighting 1 in passband and 10 in stopbands, desired passband
% group delay 20 samples
%
% om=pi*[linspace(0,.23,230),linspace(.3,.5,200),linspace(.57,1,430)];
% D=[zeros(1,230),exp(-j*om(231:430)*20),zeros(1,430)];
% W=[10*ones(1,230),ones(1,200),10*ones(1,430)];
% h = lawson(61,om,D,W,20);
%
% Author: Mathias C. Lang, Vienna University of Technology, Jan. 1998

om = om(:); D = D(:); W = W(:); L = length(om);

% Initialize, define
tol = 1e-6; h = zeros(N,1); evec = exp(-j*om);

% Initial IRLS weighting w
w = W.^2;

% Main loop
for i = 1:iter,

   % Compute weighted l2 solution
   h0 = h;
   h = lslevin(N,om,D,w);

   % Compute magnitude of complex error
   E = abs(polyval(h(N:-1:1),evec) - D);

   % Stop if nothing changes
   dh = norm(h-h0);
   if dh < tol, break; end

   % Update weight function
   update = W.*E;
   w = w.*update/(w'*update);

end   % main loop