function [r,p,kp] = lerner_lp(N,rp,fc)
% function [r,p,kp] = lerner_lp(N,rp,fc)
%
% design of digital Lerner low pass filters using the impulse invariant method
%                                    N  
% transfer function H(z) = kp * z * Sum r(n) / (z - p(n))    (parallel form)
%                                   n=1 
%
% N      filter order (min. 4)
% rp     approximate pass band ripple in dB
% fc     6dB cut-off frequencies normalized to Fs/2
% r,p,kp residues, poles and gain factor of transfer function

% G. Doblinger 12-2000, TU-Wien
   
if nargin == 0
   help lerner_lp
   return
end
if N < 4
   error('Filter order must not be less than 4');
elseif fc > 1
   error('Sampling theorem ignored');
end

% compute residues of partial fraction expansion

r = ones(1,N);
r(1) = -0.5;
if rem(N,2) == 0   % even filter order?
   r(N) = 0.5;
else
   r(N) = -0.5;
end      
r(3:2:N-1) = -1;
if rem(N, 2)       % odd filter order (low pass only)
   r = -r;         % change sign to avoid sign reversal of transfer function
end

% compute Lerner filter poles

wc = pi*fc;
nw = floor(N/2);
w = ones(1,nw);
dw = 2*wc/(N-2.5);
w(1) = -dw*(N-2)/2;
w(2:nw) = [w(1)+dw/2:dw:-dw/4];
p0 = (10^(rp/20)-1)^(dw/pi);   % rp = passband ripple in dB
p = p0*exp(j*w);

% arrange poles in complex pairs (pos. and neg. imag. part
% alternating) followed by real pole (if present)

if rem(N,2) == 0               % even filter order 
   v = ones(1,N);
   v(1:2:N) = conj(p);
   v(2:2:N) = p;
   p = v;
   v(1:2:N) = r(N:-1:nw+1);    % rearrange residues
   v(2:2:N) = r(1:nw);
   r = v;
else
   v = ones(1,N-1);
   v(1:2:N-1) = conj(p);
   v(2:2:N-1) = p;
   p = [v  p0];                % append real pole
   v(1:2:N-1) = r(N:-1:nw+2);
   v(2:2:N-1) = r(1:nw);
   r = [v r(nw+1)];
end
kp = 1/abs(sum(r./(1-p)));     % gain factor for H(e^0) = 1

%  compute and plot transfer function, and impulse response

Nf = 2048;
Nfh = Nf/2;
f = linspace(0,2,Nf);
w = exp(-j*pi*f);
H = zeros(size(w));
for n = 1:length(p)
   H = H + r(n) ./ (1 - p(n)*w);
end

H = kp*H;
h = real(ifft(H));
H = H(1:Nfh);
Hdb = 20 * log10(abs(H) + eps);
ph = unwrap(angle(H));
Tg = -Nfh / pi * diff(ph);     % Tg in samples
Nt = round(5*max(Tg));
n = 0:Nt-1;
h = h(1:Nt);
f = f(1:Nfh);
Td = sum(n.*(h.^2))/sum(h.^2);
disp(sprintf('Impulse response delay = %f samples',Td));
   
close all
if strcmp(upper(computer),'PCWIN')
   pos1 = [0.5 0.5 0.49 0.415];
   yoff = 0.5;
else	
   pos1 = [0.5 0.6 0.49 0.4];
   yoff = 0.6;
end

ftime = figure('numbertitle','off','name','Time domain','Units','normal','Position',pos1);
colordef(ftime,'black');

if Nt <= 100
   discret(n,h);
else
   plot(n,h); grid on;
end
xlabel('Time index n');
ylabel('h(n)');
title([int2str(N),' order Lerner filter impulse response']);

pos1(1) = pos1(1) - 0.495;
fmag = figure('numbertitle','off','name','Frequency domain (y-zoom an)',...
   'Units','normal','Position',pos1);
colordef(fmag,'black');

subplot(2,1,1), plot(f,20*log(abs(H)+eps)), grid on;
ylabel('Magnitude in dB');
axis([0 1 -200 20]);
title([int2str(N),' order Lerner filter transfer function']);
subplot(2,1,2), plot(f(1:Nfh-1),Tg), grid on;
xlabel('Frequency normalized to Fs/2');
ylabel('Group delay in samples');
zoom(fmag,'yon');

