function y = interpo4(x,Ng,fg)
% y = interpo4(x) or y = interpo4(x,Ng,fg)
% interpolation by factor 4 using half-band filter
% (see also interpo2.m)
%
% x   input signal
% Ng  half-band filter length = 2*Ng+1 (optional, default 30)
% fg  pass band edge of full-band filter normalized to Fs/2
%     (optional, default 0.8)
%     Note: pass band edge of half-band filter = fg/2
% y   interpolated signal of length 2*length(x)-1
%
% (c) copyright 5-1996, G.Doblinger, TU-Wien
% EMAIL: gerhard.doblinger@tuwien.ac.at

% created, Db, 5-96

% half band filter is obtained by designing full-band filter
% (with pass band edge = fg) and inserting zeros between
% samples (excluding mid-point), however zeros and mid-point
% are not used during interpolation

if nargin == 0
   help interpo4
   return
elseif nargin == 1
   Ng = 30;
   fg = 0.8;
else
   if fg >= 1
      error('fg must be less than 1');
   end
end

[nr,nc] = size(x);
if min(nr,nc) > 1
   error('x must be a vector');
end
Nx0 = max(nr,nc);
Nx = 2*Nx0-1;
h = remez(Ng-1,[0 fg 1 1],[1 1 0 0]);  % even index samples of half band filter
H = zeros(Nx-1,Nx);
Ngh = Ng/2;
Ngh1 = Ngh+1;
Ngh2 = Ngh-1;
Nxgh = Nx+Ngh;

% build interpolation matrix

for k = 1:Nx-1
   H(k,max(1,k-Ngh2):min(Nx,Ngh+k)) = h(max(1,Ngh1-k):min(Ng,Nxgh-k));
end

% perform interpolation by 2 for each pair of input samples
% (leave given samples unchanged)

x = x(:);
y = [H(1:Nx0-1,1:Nx0)*x ; 0]; % interpolated samples
y = [x.' ; y.'];              % append row of interpolated samples
x = y(:);                     % convert to column vector
x = x(1:Nx);                  % exclude last sample (0)

% repeat on new signal to obtain interpolation by 4

y = [H*x ; 0];
y = [x.' ; y.'];
y = y(:);
y = y(1:2*Nx-1);

if nc > nr
   y = y.';
end