function [Ha,Hs,Nv]=nufbark5(h80,h60,h30_20,h10_5)
% function [Ha,Hs,Nv]=nufbark5(h80,h60,h30_20,h10_5)
%
% bark scale filter bank design using qmfuni.m (unicmfb.m)
% and interpolation/decimation for nonuniform prototype filters
% (version with tight approximation of critical band analysis, 22 channels)
%
% h80...    odd length prototype low pass filter impulse responses
% Ha        analysis filter impulse responses (as colunm vectors)
% Hs        synthesis filter impulse responses (as colunm vectors)
% Nv        vector of decimation factors
%
% G. Doblinger, TU-Wien, 08-1996

% requires function interpo2.m

if nargin == 0
   help nufbark5
   return
end

L = length(h80);
if ~rem(L,2)
   error('filter h80 must have odd length');
elseif ~rem(L+1,4)
   error('L80+1 must not be divisible by 4');
end
L60 = length(h60);
if ~rem(L60,2)
   error('filter h60 must have odd length');
end
L30_20 = length(h30_20);
if ~rem(L30_20,2)
   error('filter h30_20 must have odd length');
end
L10_5 = length(h10_5);
if ~rem(L10_5,2)
   error('filter h10_5 must have odd length');
end

% find upper transition filters by interpolation with a factor 2

h60_40 = interpo2(h30_20);
L60_40 = length(h60_40);
h60_40 = sqrt(40)*h60_40/sum(h60_40);
h20_10 = interpo2(h10_5);
L20_10 = length(h20_10);
h20_10 = sqrt(10)*h20_10/sum(h20_10);

% form other impulse responses by decimation
% zero-pad impulse responses to equal length L

h40 = sqrt(2)*h80(1:2:L);
L1 = length(h40);
h40 = [zeros((L-L1)/2,1);h40;zeros((L-L1)/2,1)];
h20 = 2*h80(1:4:L);
L1 = length(h20);
h20 = [zeros((L-L1)/2,1);h20;zeros((L-L1)/2,1)];
h10 = sqrt(8)*h80(1:8:L);
L1 = length(h10);
h10 = [zeros((L-L1)/2,1);h10;zeros((L-L1)/2,1)];

h30 = sqrt(2)*h60(1:2:L60);
L1 = length(h30);
h30 = [zeros((L-L1)/2,1);h30;zeros((L-L1)/2,1)];

h60 = [zeros((L-L60)/2,1);h60;zeros((L-L60)/2,1)];

L1 = (L-L60_40)/2;
h60_40 = [zeros(L1,1);h60_40;zeros(L1,1)];
L1 = (L-L30_20)/2;
h30_20 = [zeros(L1,1);h30_20;zeros(L1,1)];
L1 = (L-L20_10)/2;
h20_10 = [zeros(L1,1);h20_10;zeros(L1,1)];
L1 = (L-L10_5)/2;
h10_5 = [zeros(L1,1);h10_5;zeros(L1,1)];

% determine frequency spacings

Lb = [4 3 4 3 4 3 1];
M = [80 60 40 30 20 10 5];
Nm = length(M);
N80 = Lb(1);

% find vector of subsampling factors Nv vector of modulation start indices kv

Nv = M(1)*ones(1,Lb(1));
kv = zeros(1,Nm-1);
it = Lb ./ M;
for n = 2:Nm
   Nv = [Nv M(n)*ones(1,Lb(n))];
   kv(n-1) = M(n) * sum(it(1:n-1));
end
kv = fix(kv);
N = length(Nv);

% modifiy vectors kv, Lb (due to trans. filters) and create prototype impulse response vector

kv = [kv(1:2),kv(2)+1,kv(3:4),kv(4)+1,kv(5),kv(5)+1,kv(6)];
Nb = [Lb(2),1,Lb(3)-1,Lb(4),1,Lb(5)-1,1,Lb(6)-1,1];
Hb = [h60,h60_40,h40,h30,h30_20,h20,h20_10,h10,h10_5];

Ha = zeros(L,N);
Hs = zeros(L,N);
N0 = (L-1)/2;
n = [0:L-1]'-N0;

% compute impulse responses of first channels

for k = 1:N80
   Nk = Nv(k);
   theta = pi/(2*Nk)*(2*k-1);
   nn = n - Nk/2;
   Ha(:,k) = h80 .* cos(theta*nn);
   nn = n + Nk/2;
   Hs(:,k) = h80 .* cos(theta*nn);
end
k0 = k+1;

% compute remaining channel impulse responses

for m = 1:length(kv)
   k1 = kv(m);
   for k = k0:k0+Nb(m)-1
      Nk = Nv(k);
      k1 = k1 + 1;
      phi = pi/(2*Nk)*(2*k1-1)*n;
      phi0 = (2*k-1)*pi/4;
      hpt = Hb(:,m);
      if any(imag(hpt))
         Ha(:,k) = real(hpt) .* cos(phi-phi0) - imag(hpt) .* sin(phi-phi0);
         Hs(:,k) = real(hpt) .* cos(phi+phi0) - imag(hpt) .* sin(phi+phi0);
      else
         Ha(:,k) = hpt .* cos(phi-phi0);
         Hs(:,k) = hpt .* cos(phi+phi0);
      end
   end
   k0 = k+1;
end

% compute reconstruction error

Nx = 2*L;
x = [1 zeros(1,Nx-1)];
y = zeros(1,Nx);
for k = 1:N
   ya = filter(2*Ha(:,k),1,x);
   m = 1:Nv(k):Nx;
   u = zeros(1,Nx);
   u(m) = ya(m);
   y = y + filter(2*Hs(:,k),1,u);
end

% compute and plot transfer function

if L < 150
   Nft = 512;
else
   Nft = 1024;
end
f = linspace(0,1,Nft);

magH = fft(Ha,2*Nft);
magH = abs(magH(1:Nft,:));
magH = magH/max(max(magH));
eps1 = 1e-6;
i = find(magH <= eps1);
if ~isempty(i)
   magH(i) = eps1 * ones(size(i));
end
magY = abs(freqz(y,1,pi*f));

if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end
pos(1) = pos(1) - 0.495;
fdom = figure('name','Frequency domain (xy-zoom on)','Units','normal','Position',pos);
figurebackcolor = 'black';
colordef(fdom,figurebackcolor);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr1 = ['Modulated filter bank transfer functions, N = ',int2str(N),', L = ',int2str(L)];
tstr2 = 'Overall transfer function of analysis+synthesis filter bank';
subplot(2,1,1), plot(f,20*log10(magH));
gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 =  title(tstr1); set(gh1,'fontsize',axfnt); grid on;
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(f,20*log10(magY(:)));
gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 =  title(tstr2); set(gh1,'fontsize',axfnt); grid on;
set(gca,'fontsize',ticfnt);
zoom(fdom,'on');

