function ht = nuftran(dp,df,h1,N1,h2,N2,Lt,plt)
% function ht = nuftran(dp,delta_f,h1,N1,h2,N2,Lt,plt)
%
% transition band pass filter design for nonuniform filterbanks
% (version for same transition band widths for h1,h2,ht)
%
% dp        design procedure, 'o' = optimization, 'l' = linear equation solving
% delta_f   transition band width normalized to Fsampl/2 (same as used for design of h1,h2)
% h1        prototype filter impulse response at lower frequencies
% N1        spacing of uniform part at lower frequencies
% h2        prototype filter impulse response at higher frequencies
% N2        spacing of uniform part at higher frequencies
% Lt        length of transition filter
% plt       1 to enable graphics output, 0 otherwise
% ht        prototype impulse response of transition filter
%
% (c) copyright 5-1996, G.Doblinger, TU-Wien

if nargin == 0
   help nuftran
   return
end
fc1 = 1/(2*N1);
fc2 = 1/(2*N2);
if df < 0 | df > fc2
   error('delta_f must be in (0,1/2N2)');
end
fs = fc2 + df;

start = clock;

h1 = h1(:)/sum(h1);
L1 = length(h1);
h2 = h2(:)/sum(h2);
L2 = length(h2);
Lv = [L1,L2,Lt];
evenodd = sum(rem(Lv,2));
if evenodd ~= 0 & evenodd ~= 3
   error('Impulse response lengths must all be even or all be odd');
end
L = max(Lv);               % common length of all filters
Lh = floor((L+1)/2);
N0 = (L-1)/2;              % delay of prototype filters
Nz1 = (L-L1)/2;            % number of zeros appended at both sides of impulse response h1
Nzt = (L-Lt)/2;

% compute constant vectors and matrices for error vector

Lfp = round(L/2);          % no. of passband freq. points
Lfs = Lfp;                 % no. of stopband freq. points
pifc1 = pi*fc1;            % pi/2N1
pifc2 = pi*fc2;            % pi/2N2
n = [Nz1:Lh-1] - N0;
theta = pifc1*linspace(0,2,Lfp).';
C1 = 2*cos(theta*n);
n = [Nzt:Lh-1] - N0;
theta = linspace(-pifc2-pifc1,-pifc2+pifc1,Lfp).';
phi = theta*n;
Ut1 = 2*cos(phi);
theta = linspace(fs*pi,pi,Lfs).';
phi = theta*n;
Uts = 2*cos(phi);
theta = linspace(-pifc2+pifc1,0,Lfp).';
phi = theta*n;
Utp = 2*cos(phi);

L1h = floor((L1+1)/2);
Lth = floor((Lt+1)/2);
if rem(L,2)
   C1(:,L1h) = ones(Lfp,1);
   Ut1(:,Lth) = ones(Lfp,1);
   Uts(:,Lth) = ones(Lfs,1);
   Utp(:,Lth) = ones(Lfp,1);
end

h1 = h1(1:L1h);
G1 = C1*h1;
a1 = G1.*G1(Lfp:-1:1);
a2 = 1 - G1.^2;
a3 = ones(Lfp,1);

% create starting solution

ht = [zeros((Lt-L2)/2,1);h2];
%ht = [zeros((Lt-L1)/2,1);h1];
ht = ht(1:Lth);

Q = Uts'*Uts;     % matrix to compute stop band energy

clear theta phi n C1 G1 h1 h2 Uts

% find optimum transition filter (low pass filter)

disp(['Transition filter design for N1 = ',int2str(N1),', N2 = ',int2str(N2),...
   ', L = ',int2str(L)]);

if strcmp(upper(dp),'L')
   alpha1 = 5;
   alpha2 = 1;
   alpha3 = 1;
   alphas = 100;
   epsil = 1e-6;                                % error limit to terminate iteration
   tau = 0.7;
   tau1 = 1-tau;
   Niter = 200;
   I = ones(1,length(ht));
   for k = 1:Niter
      d = Ut1*ht;
      B1 = (d(Lfp:-1:1)*I).*Ut1;
      B2 = (d*I).*Ut1;
      B3 = ((Utp*ht)*I).*Utp;
      U = alpha1*B1'*B1+alpha2*B2'*B2+alpha3*B3'*B3+alphas*Q;
      v = alpha1*B1'*a1+alpha2*B2'*a2+alpha3*B3'*a3;
      gt = U \ v;                               % solution for transition low pass filter
      ht = tau1*ht + tau*gt;                    % impulse response update (real part)
      err = norm(ht-gt)/norm(ht);               % impulse response error
      disp(['Iteration ', int2str(k), ', Error = ', num2str(err)]);
      if err < epsil, break, end
   end

elseif strcmp(upper(dp),'O')                    % solve nonlinear optimization problem

   Lto = length(ht);
   Niter = 200;
   alpha = [5 1 1 100];
   A = [a1 a2 a3];
   vs = version;
   if strcmp(vs(1),'5')
      options(1) = 1;                           % turn on display
      options(2) = 5e-5;                        % termination criterion for variable h
      options(3) = 5e-5;                        % termination criterion for cost func. (f)
      options(6) = 1;                           % DFP-algorithm for fminu()
      options(14) = Niter;                      % maximum number of iterations
      ht = fminu('nuf_er',ht,options,'nuf_grd',Lfp,Lto,A,Ut1,Utp,Q,alpha);
   else                                         % new optimization toolbox with MATLAB 6.0
      options = optimset('Display','on','TolX',5e-5,'TolFun',5e-5,...
         'MaxIter',Niter,'LargeScale','off','GradObj','on');
      ht = fminunc('nuf_er_grd',ht,options,Lfp,Lto,A,Ut1,Utp,Q,alpha);
   end

else
   error('no such optimization procedure');
end

% form symmetric prototype low pass filter impulse response

j = sqrt(-1);
if rem(L,2)
   ht = sqrt(N2) * [ht ; ht(Lth-1:-1:1)];
else
   ht = sqrt(N2) * [ht ; ht(Lth:-1:1)];
end

disp(sprintf('\n Elapsed time = %f sec.', etime(clock,start)));

% compute and plot transfer function

if plt
   Nft = 512;
   f = linspace(0,1,Nft);

   magH = abs(fft(ht,2*Nft));
   magH = magH(1:Nft);
   magH = magH/max(max(magH));
   eps1 = 1e-6;
   i = find(magH <= eps1);
   if ~isempty(i)
      magH(i) = eps1 * ones(size(i));
   end
   close all
   if strcmp(upper(computer),'PCWIN')
      pos = [0.5 0.5 0.49 0.415];
      axfnt = 8;
      ticfnt = 7;
   else	
      pos = [0.5 0.6 0.49 0.4];
      axfnt = 12;
      ticfnt = 10;
   end
   fdom = figure('name','Frequency domain (xy-zoom on)','Units','normal','Position',pos);
   figurebackcolor = 'black';
   colordef(fdom,figurebackcolor);

   xstr = 'f normalized to Fsample';
   ystr = 'Magnitude in dB';
   tstr = ['Transition prototype filter transfer function, Lt = ',int2str(Lt)];
   plot(f,20*log10(magH))
   gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
   gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
   gh1 =  title(tstr); set(gh1,'fontsize',axfnt); grid on;
   set(gca,'fontsize',ticfnt);
   grid on;
   zoom(fdom,'on');
end