function nufplot(h1,N1,h2,N2,ht,Nbp1,finv)
% function nufplot(h1,N1,h2,N2,ht,Nbp1,finv)
%
% plot nonuniform (N1,N2) filter bank design result with N1 > N2
% (i.e. smaller bandwidth at low frequencies)
%
% h1        prototype filter impulse response at lower frequencies
% N1        spacing of uniform part at lower frequencies
% h2        prototype filter impulse response at higher frequencies
% N2        spacing of uniform part at higher frequencies (N1
%           divisible by N2)
% ht        prototype impulse response of transition filter
% Nbp1      number of equal lower band channels (Nbp1 < N1, Nbp1
%           divisible by N1/N2)
% finv      if 1, then inversion of frequency axis is carried out by
%           multiplying channel impulse responses by (-1)^n
%           (i.e. smaller bandwidth at high frequencies)
%
% G. Doblinger, TU-Wien, 03-1996

if nargin == 0
   help nufplot
   return
end
if N1 > N2
   Nbp2 = N2-1-Nbp1*N2/N1;
   if rem(N1,N2)
      error('N1 must be divisible by N2')
   elseif rem(Nbp1,N1/N2)
      error('Nbp1 must be divisible by N1/N2')
   elseif Nbp2 < 1
      error('Nbp1 to large')
   end
else
   error('N1 must be greater than N2');
end

fc1 = 1/(2*N1);
fc2 = 1/(2*N2);
L1 = length(h1);
L2 = length(h2);
Lt = length(ht);
L = max([L1,L2,Lt]);       % common length of all filters
N0 = (L-1)/2;              % delay of prototype filters
Nz1 = (L-L1)/2;            % number of zeros appended at both sides of impulse response h1
Nz2 = (L-L2)/2;
Nzt = (L-Lt)/2;

% zero-padd prototype impulse responses to length L

h1 = 2*sqrt(N1)*h1(:)/sum(h1);
h2 = 2*sqrt(N2)*h2(:)/sum(h2);
ht = 2*sqrt(N2)*ht(:)/sum(ht);
h1 = [zeros(Nz1,1) ; h1 ; zeros(Nz1,1)];
h2 = [zeros(Nz2,1) ; h2 ; zeros(Nz2,1)];
ht = [zeros(Nzt,1) ; ht ; zeros(Nzt,1)];

% compute modulation matrices (for channel impulse responses)

n = [0:L-1]'-N1/2-N0;
i = 2*[1:Nbp1]-1;
I = n*i;
Ma1 = cos(pi*fc1*I);
n = n + N1;
I = n*i;
Ms1 = cos(pi*fc1*I);
hlp = h1 * ones(1,Nbp1);
h1a = hlp .* Ma1;                  % analysis filters
g1s = hlp .* Ms1;                  % synthesis filters

n = [0:L-1]'-N2/2-N0;
it = 2*(N2-Nbp2)-1;
i = 2*[N2-Nbp2+1:N2]-1;
I = n*i;
Ma2 = cos(pi*fc2*I);
j = sqrt(-1);
Mat = exp(j*pi*fc2*it*n);
n = n + N2;
I = n*i;
Ms2 = cos(pi*fc2*I);
Mst = exp(j*pi*fc2*it*n);
hlp = h2 * ones(1,Nbp2);
h2a = hlp .* Ma2;
g2s = hlp .* Ms2;
hta = real(ht.*Mat);
gts = real(ht.*Mst);

h = [h1a hta h2a];
g = [g1s gts g2s];

if finv     % frequency inversion
   h(2:2:length(h),:) = -h(2:2:length(h),:);
   g(2:2:length(g),:) = -g(2:2:length(g),:);
end

% compute reconstruction error

N = Nbp1 + Nbp2 + 1;
Nx = 2*L;
x = [1 zeros(1,Nx-1)];
Y = zeros(N,Nx);
for k = 1:N
   Y(k,:) = filter(h(:,k),1,x);
end
U = zeros(size(Y));
m = 1:N1:Nx;
k = 1:Nbp1;
U(k,m) = Y(k,m);
m = 1:N2:Nx;
k = Nbp1+1:N;
U(k,m) = Y(k,m);
y = zeros(1,Nx);
for k = 1:N
   y = y + filter(g(:,k),1,U(k,:));
end

% compute and plot transfer function

Nft = 512;
f = linspace(0,1,Nft);

magH = fft(h,2*Nft);
magH = abs(magH(1:Nft,:));
magH = magH/max(max(magH));
eps1 = 1e-6;
i = find(magH <= eps1);
if ~isempty(i)
   magH(i) = eps1 * ones(size(i));
end
magY = abs(freqz(y,1,pi*f));

if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end
fdom = figure('name','Frequency domain (xy-zoom on)','Units','normal','Position',pos);
figurebackcolor = 'black';
colordef(fdom,figurebackcolor);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr1 = ['Cos-modulated filter bank transfer functions, N = ',int2str(N),', L = ',int2str(L)];
tstr2 = 'Overall transfer function';
subplot(2,1,1), plot(f(:),20*log10(magH));
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 =  title(tstr1); set(gh1,'fontsize',axfnt); grid on;
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(f(:),20*log10(magY(:)));
gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 =  title(tstr2); set(gh1,'fontsize',axfnt); grid on;
set(gca,'fontsize',ticfnt);
zoom(fdom,'on');
