function qmf_2lat(alpha)
% function qmf_2lat(alpha)
%
% plot design result 2-channel orthonormal lattice QMF-bank
%
% alpha  lattice coefficient vector obtained with function QMF_Lat_Coef()
%
% G. Doblinger, 01-1998

% ref.: P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

if nargin == 0
   help qmf_2lat
   return
end

% compute QMF lattice filter impulse responses

N = 2*length(alpha);
x = [1 ; zeros(N-1,1)];
[h1,h2] = QMFaLattice(x,0,alpha);

% compute reconstruction error

Nx = 1024;
Nx2 = Nx/2;
x = perinois(Nx,Nx2,[],sqrt(2));                   % create periodic white noise
[y_lp,y_hp] = QMFaLattice(x,1,alpha);
y = QMFsLattice(y_lp,y_hp,1,alpha);
y = y(Nx2+1:Nx);

% compute and plot transfer function

f = linspace(0,1,Nx2);
fy = linspace(0,1,Nx2/2);

magH1_l = abs(freqz(h1,1,pi*f));
magH1_l = magH1_l/max(magH1_l);
eps1 = 1e-6;
i = find(magH1_l <= eps1);
if ~isempty(i)
   magH1_l(i) = eps1 * ones(size(i));
end
magH2_l = abs(freqz(h2,1,pi*f));
magH2_l = magH2_l/max(magH2_l);
i = find(magH2_l <= eps1);
if ~isempty(i)
   magH2_l(i) = eps1 * ones(size(i));
end
magY_l = fft(y);
magY_l = abs(magY_l(1:Nx2/2));

i = find(magY_l <= eps1);
if ~isempty(i)
   magY_l(i) = eps1 * ones(size(i));
end

close all
if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end
ftd = figure('name','Time domain','Units','normal','Position',pos);
figurebackcolor = 'black';
colordef(ftd,figurebackcolor);

xstr = 'Time index n';
n = [0:N-1];
Nd_lp = round(sum(n'.*(h1.^2))/sum(h1.^2));   % energy delays
Nd_hp = N - Nd_lp - 1;
ystr1 = 'h_{lp}(n)';
ystr2 = 'h_{hp}(n)';
tstr1 = ['Analysis LP filter, N_{lp} = ',int2str(N),...
         ', energy delay = ', num2str(Nd_lp), ' samples'];
tstr2 = ['Analysis HP filter, N_{hp} = ',int2str(N),...
         ', energy delay = ', num2str(Nd_hp), ' samples'];
subplot(2,1,1), discret(n,h1);
h1 = ylabel(ystr1); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), discret(n,h2);
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr2); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);

pos(1) = pos(1) - 0.495;  % position of next figure
ftf = figure('name','Frequency domain (y-zoom on)','Units','normal','Position',pos);
colordef(ftf,figurebackcolor);

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr1 = 'QMF-bank transfer functions (orthonormal lattice filters)';
tstr2 = ['Overall transfer function, filter bank delay = ',...
         int2str(N-1), ' samples'];
subplot(2,1,1), plot(f,20*log10(magH1_l),'y',f,20*log10(magH2_l),'m');
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr1); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(fy,20*log10(magY_l),'y');
h1 = xlabel(xstr); set(h1,'fontsize',axfnt);
h1 = ylabel(ystr); set(h1,'fontsize',axfnt);
h1 = title(tstr2); set(h1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
zoom(ftf,'yon');

% --------------------------------------------------------------------------------------

function [y_lp,y_hp] = QMFaLattice(x,dec,alpha)
%function [y_lp,y_hp] = QMFaLattice(x,dec,alpha)
%
% compute response of 2-channel QMF lattice analysis filter bank
%
% x            input signal vector
% dec          if 0, then no downsampling is applied
%                 1, downsampling by factor 2
% alpha        vector of lattice coefficients computed with Fir2Lattice()
% y_lp, y_hp   output signal vectors of low pass and high pass filters

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

[Nxr,Nxc] = size(x);
if min(Nxr,Nxc) > 1
   error('QMFaLattice: input signal must not be a matrix');
end
Nx = max(Nxr,Nxc);
N = length(alpha);

% compute lattice input signals

p0 = x(:)/sqrt(2);
q0 = [0 ; p0(1:Nx-1)];
if dec
   p0 = p0(2:2:Nx);
   q0 = q0(2:2:Nx);
   zpad = 0;
   Nq = length(q0)-1;
else
   zpad = [0 ; 0];
   Nq = Nx-2;
end

% compute first stage separately

p = p0 - alpha(1)*q0;
q = q0 + alpha(1)*p0;

for n = 2:N
   q = [zpad ; q(1:Nq)];   % delayed q-vector
   pt = p;
   p = p - alpha(n)*q;
   q = q + alpha(n)*pt;
end

beta = 1/sqrt(prod(1+alpha.^2));
y_lp = beta*p;
y_hp = beta*q;
if Nxr == 1
   y_lp = y_lp.';
   y_hp = y_hp.';
end

% --------------------------------------------------------------------------------------

function y = QMFsLattice(x_lp,x_hp,int,alpha)
%function y = QMFsLattice(x_lp,x_hp,int,alpha)
%
% compute response of 2-channel QMF lattice synthesis filter bank
%
% x_lp, x_hp   input signal vectors computed with QMFaLattice()
% int          if 0, then no upsampling is applied
%                 1, upsampling by factor 2
% alpha        vector of lattice coefficients computed with Fir2Lattice()
% y            output signal vector

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

[Nxr,Nxc] = size(x_lp);
if min(Nxr,Nxc) > 1
   error('QMFsLattice: input signal must not be a matrix');
end
Nx = max(Nxr,Nxc);
N = length(alpha);
alpha = alpha(N:-1:1);     % use reversed coefficients

if int
   zpad = 0;
   Nq = Nx-1;
else
   zpad = [0 ; 0];
   Nq = Nx-2;
end

% compute first stage separately

x_lp = x_lp(:);
x_hp = x_hp(:);
p = x_lp + alpha(1)*x_hp;
q = x_hp - alpha(1)*x_lp;

for n = 2:N
   p = [zpad ; p(1:Nq)];   % delayed p-vector
   pt = p;
   p = p + alpha(n)*q;
   q = q - alpha(n)*pt;
end

if int
   Nx = 2*Nx;
   p_int = zeros(Nx,1);
   p_int(2:2:Nx) = p;
   q_int = zeros(Nx,1);
   q_int(2:2:Nx) = q;
   p = 2*p_int;
   q = 2*q_int;
end
p = [0 ; p(1:Nx-1)];

y = (p + q)/sqrt(2*prod(1+alpha.^2));

if Nxr == 1
   y = y.';
end
