function [y,xn] = wt_dn(x,N,SNR,thr,h)
% [y,xn] = wt_dn(x,N,SNR,thr,h)
%
% WT denoising using polyphase 2-channel QMF filter banks
% for linear phase filters, and lattice filters for
% orthonormal wavelet filters
%
% x    input signal vector (length > 2^N+length(h))
% N    number of channels 
% SNR  SNR of additive white gaussian noise
% thr  if 's', then use soft thresholding of DTWT,
%      if 'h' use hard thresholding, 
%      else (or omitted) no thresholding 
% h    prototype low-pass filter response (or lattice filter
%      coefficients), use coif4, if omitted
% y    output of DTWT + IDTWT (delay compensated)
% xn   noisy input signal
%
% example:  h_lp = qmf_2(50,0.65,0.1);
%           xc = sin(4e-5*[0:2047].^2);
%           [y,xn] = wt_dn(xc,6,15,'h',h_lp);
% or simply [y,xn] = wt_dn(xc,6,15,'h');
%
% G. Doblinger, TU-Wien, 2-2001

% lattice coefficients for a Coiflet

alpha = [ -1.8259736695136908e+000
  6.8624017458191322e-001
 -1.2936373135391148e+000
  7.6545290427557378e+000
  5.8325345941061624e+000
 -3.3850833006579335e+000
  1.2385800701844751e+000
 -3.1984994217392476e-001
  8.8686114540988667e-002
 -7.8950962788019524e-002
  1.8530043529985014e-002
  4.9989981262503227e+002];

if nargin == 0
   help wt_dn
   return
elseif nargin == 2
   SNR = inf;
   h = alpha;
elseif nargin == 3
   h = alpha;
   thr = '';
elseif nargin == 4
   h = alpha;
end

h = h(:);
L  = length(h);
Nd =  sum([0:L-1]'.*(h.^2))/sum(h.^2);   % energy delay
if abs(Nd - (L-1)/2) < 1e-3
   fistr = ', polyphase filters';
   htype = 1;
else
   fistr = ', lattice filters'; 
   htype = 0;
end
N = max(2,N);

% TP and HP filter impulse responses

if htype == 1
   p1 = h(1:2:end);                      % polyphase filters 
   p2 = h(2:2:end);   
end

% select input signal, add noise

Nx = length(x);
if Nx < 2^N + L
   error('length of x must be greater than 2^N + length(h)');
end
x = x(:);
if SNR ~= inf
   v = randn(size(x));
   v = v - mean(v);
   sigma_x = std(x);
   if sigma_x <= eps                     % DC signal
      sigma_x = mean(x) + eps;
   end
   v = sigma_x/std(v)*10^(-SNR/20)*v;
   sigma_v = std(v);
   xn = x + v;                           % noisy signal
else
   xn = x;
end

% analysis filter bank output signals stored in matrix Y

Y = zeros(Nx+L,N);
yl = xn;
if htype == 1
   for n = 1:N-1
      [yl,yh] = polyafib2(p1,p2,yl);
      Y(1:length(yh),n) = yh;
   end
else
   for n = 1:N-1
      [yl,yh] = QMFaLattice(h,yl);
      Y(1:length(yh),n) = yh;
   end
end
Y(1:length(yl),N) = yl;

% apply thresholding to wavelet coefficients

if SNR ~= inf
   threshold = sigma_v*sqrt(2*log(Nx));  % global threshold estimate
   Y1(:,N) = Y(:,N);                     % keep Y(:,N) ( DTWT at highest scale)
   nchan = 1:N-1;                        % modify all others                       
   if strcmp(upper(thr),'S')
      Y1(:,nchan) = sign(Y(:,nchan)) .*  max(0,abs(Y(:,nchan))-threshold); % soft threshold
      thstr = ', soft';
   elseif strcmp(upper(thr),'H')
      Y1(:,nchan) = Y(:,nchan) .* (abs(Y(:,nchan)) > threshold);           % hard threshold
      thstr = ', hard';
   else
      Y1 = Y;
      thstr = ', no';
   end
else
   Y1 = Y;
   thstr = ', no';
end
 
% synthesis filter bank

y = Y1(1:length(yl),N);
if htype == 1
   for n = N-1:-1:1
      y = polysfib2(p1,p2,y,Y1(1:length(y),n));
   end
else
   for n = N-1:-1:1
      y = QMFsLattice(h,y,Y1(1:length(y),n));
   end
end
y = y(1:Nx);

% plot results

close all
if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end

fdt = figure('name','Time domain (x-zoom on','Units','normal','Position',pos);
figurebackcolor = 'black';
colordef(fdt,figurebackcolor);

n = [0:Nx-1];
subplot(2,1,1), plot(n,xn);
gh = ylabel('x(n)+w(n)'); set(gh,'fontsize',axfnt);
gh = title(['Noisy input signal, SNR = ',num2str(SNR),' dB']); set(gh,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
subplot(2,1,2), plot(n,y);
gh = xlabel('Time index n'); set(gh,'fontsize',axfnt);
gh = ylabel('y(n)'); set(gh,'fontsize',axfnt);
gh = title(['Denoised signal, DTWT N = ',int2str(N),thstr,' thresholding',...
	   fistr,' L = ',int2str(L)]); set(gh,'fontsize',axfnt);
set(gca,'fontsize',ticfnt);
zoom(fdt,'xon');

% plot traces of wavelet coefficients

pos(1) = pos(1) - 0.495;
ft = figure('name','Plot of DTWT coefficients (x-zoom on)','Units','normal',...
   'Position',pos);
colordef(ft,figurebackcolor);

YTLmat = [];
RGBmat = [];
redstep = ones(1,N);
greenstep = linspace(0,1,N);
bluestep = linspace(1,0,N);
for n = 1:N
   YTLmat = [YTLmat ; sprintf('DTWT(%2d,n)',n)];
   RGBmat =  [RGBmat ; [redstep(n),greenstep(n),bluestep(n)]];
end

NY = round((Nx+L)/2);
Y1 = Y1(1:NY,:);
Y1 = Y1/max(max(abs(Y1)));
Y1 = Y1 + ones(NY,1)*[1:N];           % add offset for plotting
k = 1:NY;
tstr = 'Normalized DTWT coefficients';
axh = axes('Box','on','YTick',[1:N],'YTickLabel',YTLmat,'XLim',[0,NY],'Ylim',[0,N+1]);
for n = 1:N
   line(k,Y1(:,n),'Color',RGBmat(n,:));
end
set(axh,'FontSize',axfnt);
set(gca,'xlabel',text(0.5,-0.1,'Index n','fontsize',axfnt,...
   'Units','normal','HorizontalAlignment','center'));
set(gca,'title',text(0.5,1.05,tstr,'fontsize',axfnt,...
   'Units','normal','HorizontalAlignment','center'));
set(gca,'fontsize',ticfnt);
zoom(ft,'xon');

% --------------------------------------------------------------------

function [yl,yh] = polyafib2(p1,p2,x)

% compute analysis filter bank (polyphase filters)

u1 = conv(p1,x(1:2:end));
u2 = conv(p2,[0 ; x(2:2:end-1)]);
yl = u1 + u2;
yh = u1 - u2;

% --------------------------------------------------------------------

function y = polysfib2(p1,p2,yl,yh)

% compute synthesis filter bank (polyphase filters)

L = 2*length(p1);
Ly = 2*length(yl)+L-2;
v1 = yl - yh;
v2 = yl + yh;
y = zeros(Ly,1);
y(1:2:Ly) = conv(p1,v1);
y(2:2:Ly) = conv(p2,v2); 
y = y(L:Ly+3-L);   % compensate delay of 2 channel analysis/synthesis filter bank

% --------------------------------------------------------------------


function [yl,yh] = QMFaLattice(alpha,x)
%
% compute response of decimated 2-channel QMF lattice analysis filter bank
%
% alpha    vector of lattice coefficients computed with Fir2Lattice()
% x        input signal vector
% yl, yh   output signal vectors of low pass and high pass filters

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

Nx = length(x);
N = length(alpha);
Ny = floor((Nx-1)/2+N); % required output signal length

% compute decimated lattice input signals

q0 = x(1:2:Nx-1);       % down-sampling commutator
p0 = x(2:2:Nx);
zpad = zeros(Ny-length(p0),1);
p0 = [p0(:) ; zpad];    % zero padding
q0 = [q0(:) ; zpad];
Nq = length(q0)-1;

% compute first stage separately

p = p0 - alpha(1)*q0;
q = q0 + alpha(1)*p0;

for n = 2:N
   q = [0 ; q(1:Nq)];   % delayed q-vector
   pt = p;
   p = p - alpha(n)*q;
   q = q + alpha(n)*pt;
end

beta = 1/sqrt(prod(1+alpha.^2));
yl = beta*p;
yh = beta*q;

% --------------------------------------------------------------------

function y = QMFsLattice(alpha,xl,xh)
%
% compute response of 2-channel QMF lattice synthesis filter bank
%
% alpha    vector of lattice coefficients computed with Fir2Lattice()
% xl, xh   input signal vectors computed with QMFaLattice()
% y        output signal vector

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, P.Hoang, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

N = length(alpha);
Nx = length(xl) + N;
Nq = Nx - 1;
alpha = alpha(N:-1:1);  % use reversed coefficients

% compute first stage separately

xl = [xl(:) ; zeros(N,1)];
xh = [xh(:) ; zeros(N,1)];
p = xl + alpha(1)*xh;
q = xh - alpha(1)*xl;

for n = 2:N
   p = [0 ; p(1:Nq)];   % delayed p-vector
   pt = p;
   p = p + alpha(n)*q;
   q = q - alpha(n)*pt;
end

% up-sampling commutator

Nx = 2*Nx;
y = zeros(Nx,1);
y(1:2:Nx) = q;
y(2:2:Nx) = p;
y = y(2*N-1:Nx-2*N)/sqrt(prod(1+alpha.^2));   % compensate filter gain and delay
