function wtfib(N,h_lo,h_hi)
%function wtfib(N,h_lo,h_hi)
%
% compute non-decimated filter transfer functions of DTWT channels
%
% N   number of WT channels
% h_lo, h_hi  low pass and high pass filter impulse resonses
%             if h_hi is omitted, then lattice filters ar assumed,
%             and h_lo = vector of lattice coefficients
%
% G. Doblinger, TU-Wien, 03-2001

if nargin == 0
   help wtfib
   return
elseif N >= 10
   disp('N to large');
   return
end

if nargin == 2                % Lattice filters
   alpha = h_lo;
   [h_lo,h_hi] = QMFaLattice([1 ; zeros(2*length(alpha)-1,1)],0,alpha);
end
h_lo = h_lo(:);
h_hi = h_hi(:);
N = max(N,2);
L = length(h_lo);
NX = (2^(N-1)-1)*L;           % maximum length = length of iterated
                              % low pass filter impulse response 
X = zeros(NX,N);              % matrix of channel impulse responses
X(1:L,N) = h_hi;              % channel N = high pass
if N == 2
   X(:,1) = h_lo;
else
   M = 1;
   g = h_lo(:);
   for n = N-1:-1:2           % iterate filters
      M = 2*M;
      w = zeros(M*L,1);
      w(1:M:end) = h_hi;      % shift decimation through filter
                              % (z -> z^M by interpolating with zeros)
      h = conv(g,w);
      X(1:length(h),n) = h;   % band pass filters
      w(1:M:end) = h_lo;
      g = conv(g,w);          % iterate low pass filter
   end
   NX = length(g);
   X(1:NX,1) = g;             % channel 1 = iterated low pass
                              % (g = scaling function for large N)
   X = X(1:NX,:);			      
end

% compute frequency responses

Nf = max(1024,2^ceil(log2(NX)));
Xf = fft(X,Nf);
Nf = Nf/2+1;
Xf = Xf(1:Nf,:);
Xf = max(-100,20*log10(abs(Xf)+eps));
f = linspace(0,1,Nf);

close all
if strcmp(upper(computer),'PCWIN')
   pos = [0.5 0.5 0.49 0.415];
   axfnt = 8;
   ticfnt = 7;
else	
   pos = [0.5 0.6 0.49 0.4];
   axfnt = 12;
   ticfnt = 10;
end

% plot channel filter impulse responses

ft = figure('name','Time domain (x-zoom on)','Units','normal',...
   'Position',pos);
colordef(ft,'black');

YTLmat = [];
RGBmat = [];
redstep = ones(1,N);
greenstep = linspace(0,1,N);
bluestep = linspace(1,0,N);
for n = 1:N
   YTLmat = [YTLmat ; sprintf('h(%2d,n)',n)];
   RGBmat =  [RGBmat ; [redstep(n),greenstep(n),bluestep(n)]];
end

X1 = X/max(max(abs(X)));
X1 = X1 + ones(NX,1)*[1:N];           % add offset for plotting
k = 1:NX;
tstr = 'Channel filter impulse responses';
axh = axes('Box','on','YTick',[1:N],'YTickLabel',YTLmat,'XLim',[0,NX],'Ylim',[0,N+1]);
for n = 1:N
   line(k,X1(:,n),'Color',RGBmat(n,:));
end
set(axh,'FontSize',axfnt);
set(gca,'xlabel',text(0.5,-0.1,'Index n','fontsize',axfnt,...
   'Units','normal','HorizontalAlignment','center'));
set(gca,'title',text(0.5,1.05,tstr,'fontsize',axfnt,...
   'Units','normal','HorizontalAlignment','center'));
set(gca,'fontsize',ticfnt);
zoom(ft,'xon');

pos(1) = pos(1) - 0.495;
ftf = figure('name','Frequency domain (y-zoom on)','Units','normal','Position',pos);
colordef(ftf,'black');

xstr = 'f normalized to Fsample/2';
ystr = 'Magnitude in dB';
tstr = ['nondecimated DTWT channel filters, N = ',int2str(N)];
set(gca,'NextPlot','replacechildren');
set(gca,'Colororder',RGBmat);
plot(f,Xf)
gh1 = xlabel(xstr); set(gh1,'fontsize',axfnt);
gh1 = ylabel(ystr); set(gh1,'fontsize',axfnt);
gh1 = title(tstr); set(gh1,'fontsize',axfnt); grid on
set(gca,'fontsize',ticfnt);
%set(gca,'PlotBoxAspectRatio',[1.8 1 1]);
zoom(ftf,'yon');

% --------------------------------------------------------------------------------------

function [y_lp,y_hp] = QMFaLattice(x,dec,alpha)
%function [y_lp,y_hp] = QMFaLattice(x,dec,alpha)
%
% compute response of 2-channel QMF lattice analysis filter bank
%
% x            input signal vector
% dec          if 0, then no downsampling is applied
%                 1, downsampling by factor 2
% alpha        vector of lattice coefficients computed with Fir2Lattice()
% y_lp, y_hp   output signal vectors of low pass and high pass filters

% Db, 6.1.98
% ref.: P.P.Vaidyanathan, "Lattice structures for optimal design and robust
%     implementation of two-channel perfect-reconstruction QMF banks", IEEE, Trans.
%     ASSP, Vol. 36, pp.81-94, Jan. 1988.

[Nxr,Nxc] = size(x);
if min(Nxr,Nxc) > 1
   error('QMFaLattice: input signal must not be a matrix');
end
Nx = max(Nxr,Nxc);
N = length(alpha);

% compute lattice input signals

p0 = x(:);
q0 = [0 ; p0(1:Nx-1)];
if dec
   p0 = p0(2:2:Nx);
   q0 = q0(2:2:Nx);
   zpad = 0;
   Nq = length(q0)-1;
else
   zpad = [0 ; 0];
   Nq = Nx-2;
end

% compute first stage separately

p = p0 - alpha(1)*q0;
q = q0 + alpha(1)*p0;

for n = 2:N
   q = [zpad ; q(1:Nq)];   % delayed q-vector
   pt = p;
   p = p - alpha(n)*q;
   q = q + alpha(n)*pt;
end

beta = 1/sqrt(prod(1+alpha.^2));
y_lp = beta*p;
y_hp = beta*q;
if Nxr == 1
   y_lp = y_lp.';
   y_hp = y_hp.';
end

