function y = enhance(x,w,N,alpha,rho,Aw)
% function  y = enhance(x,w,N,alpha,rho,Aw)
%
% speech enhancement using FFT-filter bank and  Ephraim-Malah gain curves
%
% x         noisy real-valued input signal
% w         noise sequence used to estimate spectral power density 
% N         time window length
% alpha     parameter in [0,1] for a priori SNR estimate (typ. 0.985)
% rho       minimum value of a priori parameter r = SNR/(1+SNR) (typ. 0.015)
% Aw        gain for noise spectral power density (typ. 1...1.2)
% y         de-noised signal
%
% G. Doblinger, TU-Wien, 12-1994

if nargin == 0
   help enhance
   return
end
if alpha < 0 | alpha > 1
   error('alpha must be in [0,1]');
end
alpha1 = 1-alpha;

M = round(N/4);           % decimation factor
h = hanning(N);           % time window function
h = 2*sqrt(M/N)*h(:);
Nf = 2^ceil(log2(N));     % FFT length (power of 2 >= N)
Nh = Nf/2+1;
x = [x(:) ; zeros(N-length(x),1)];  % append zeros, if necessary
Nx = length(x);
w = [w(:) ; zeros(N-length(w),1)];
Nw = length(w);


% compute gain factor matrix G of Ephraim-Malah gain curves

Nrp = 18;                 % number of gain curves
Nr = 128;                 % number of points per gain curve
Nr1 = Nr-1;
Rprio_min_dB = -30;       % min, max dB-values for Rprio
Rprio_max_dB = 30;
Rprio_min = 10^(Rprio_min_dB/10);
Rprio_max = 10^(Rprio_max_dB/10);
Rpost = [0.1;0.3;0.7;1;1.5;2;2.5;3;4;5;6;8;10;12;14;16;20;30];
r = linspace(Rprio_min/(1+Rprio_min),Rprio_max/(1+Rprio_max),Nr);
                          % r = Rprio/(1+Rprio)
V = (1+Rpost)*r;
G = 0.5*sqrt(pi*(1./(1+Rpost))*r).*exp(-0.5*V).*real((1+V).*besseli(0,0.5*V)+...
						  V.*besseli(1,0.5*V));
G = G/max(max(G'));       

% compute estimates of noise spectral power densities

Pw = zeros(Nh,1);
for m = 1:M:Nw-N+1                  % frame index loop
   X = fft(w(m:m+N-1).*h,Nf);
   Pw = Pw + abs(X(1:Nh)).^2;       % sum of squared magnitudes
end
Nfr = ceil(Nw/M);
Pw = Aw^2/Nfr*Pw + eps;             % apply gain and number of frames
y = zeros(Nx,1);
gain = zeros(Nh,1);
Px = zeros(Nh,1);
I = ones(Nrp,1)*[1:Nh];             % index matrix for SNR
%Rpost = Rpost*ones(1,Nh);           % matrix of Rpost values as columns

% enhancement frame loop

for m = 1:M:Nx-N+1

   % analysis filter bank

   X = fft(x(m:m+N-1).*h,Nf);

   % compute gain factor, modify STFT

   Pprio = alpha*Px.*gain.^2;
   Px = abs(X(1:Nh)).^2;            % new STFT squared magnitude
   Ppost = max(0,Px - Pw);
   Pprio = Pprio + alpha1*Ppost;    % estimated power for a priori SNR
   r = Pprio ./ (Pprio + Pw);
   r = max(r,rho);
   ir = 1 + round(Nr1*r);           % column index for gain matrix
   SNR = Ppost ./ Pw;               % estimated a posteriori SNR

%   [dum,ipo] = min(abs(SNR(I)-Rpost));
%   gain = diag(G(ipo,ir));
   for k=1:Nh
      [dum,ipo] = min(abs(SNR(k)-Rpost));  % find index of nearest Rpost table entry
                                           % (i.e. row index of gain matrix)
      gain(k) = G(ipo,ir(k));
   end

   X = X .* [gain ; gain(Nh-1:-1:2)];      % apply gain factor
   y1 = real(ifft(X,Nf));                  % DFT synthesis filter bank
   y(m:m+N-1) = y(m:m+N-1) + y1(1:N).*h;
end
