function y = time_scale(x,scale,N)
% function  y = time_scale(x,scale,N)
%
% time scaling of audio signals using an FFT filter bank
%
% x      real-valued input signal (row, or column vector)
% scale  pitch scale factor in [0.5,2]
% N      time window length
% y      output signal
%
% G. Doblinger, TU-Wien, 02-2001

M = round(N/8);          % decimation factor
Ms = round(scale*M);     % synthesis stage decimation factor
disp(sprintf('actual scaling factor = %d/%d = %f', Ms, M, Ms/M));

h = hanning(N);          % time window function
h = 2*sqrt(sqrt(M*Ms)/N)*h(:);

Nf =  2^ceil(log2(N));   % FFT length, power of 2 >= time window length 
Nh = Nf/2+1;
x = [x(:) ; zeros(N-length(x),1)];  % append zeros, if necessary
nsig = length(x);

% initialize pitch scaling variables

ana_phase = zeros(Nh,1);
syn_phase = zeros(Nh,1);
freq = 2*pi/N*[0:Nh-1]';
c1 = 1/M;
diffph0 = M*freq;
Ny = N+ceil((nsig-N)/M*Ms);      % length of output signal
y = zeros(Ny,1);
ms = 1;                          % ms = synthesis stage frame index 

for m = 1:M:nsig-N+1             % analysis frame index loop

   % analysis filter bank

   X = fft(x(m:m+N-1).*h,Nf);
   X = X(1:Nh);

   % compute instantaneous phase ana_phase
   
   ph = atan2(imag(X),real(X));
   diffph = ph - ana_phase - diffph0;
   ana_phase = ph;
   
   % limit differential phase to [-pi,pi]
   
   diffph = rem(diffph,2*pi);
   ii = find(diffph > pi);
   diffph(ii) = diffph(ii) - 2*pi;
   ii = find(diffph < -pi);
   diffph(ii) = diffph(ii) + 2*pi;

   % compute instantaneous frequencies
   
   inst_freq = freq + c1*diffph;
      
   % compute synthesis phase and modified spectrum
   
   syn_phase = rem(syn_phase + Ms*inst_freq,2*pi);
   X = abs(X) .* exp(j*syn_phase);
   X = [X ; conj(X(Nh-1:-1:2))];
   
   % synthesis filter bank

   y1 = real(ifft(X,Nf));
   y(ms:ms+N-1) = y(ms:ms+N-1) + y1(1:N).*h;
   ms = ms + Ms;                 % update synthesis stage frame index 
end
