  function spect(x,dB,N,M,pos,signame)
% function spect(x,dB,N,M,pos,signame)
%
% call: spect(x,dB,N,M,pos,signame)
%
% compute and display spectrogram (0,Fs/2), Hamming time window
%
% x         signal vector
% dB        dynamic range of dB scale (linear scale, if dB = [])
% N         time window length (typ. N=128)
% M         decimation factor for time scale (typ, M=N/4)
%           (determines overlapping M/N of time frames)
% pos       figure window position
% signame   string of signal name
%
% G. Doblinger, TU Wien, 08-1998

global G_Fsampl G_x_zoom         % used in specplay.m

if nargin == 0
   help spect
   return
end

if min(size(x)) ~= 1
   error('x must be a vector')
end

w = hamming(N);                     % use Hamming window for time signal weighting
w = w(:);
x = x(:);
Nx = length(x);
x = [x;zeros(N-Nx,1)];              % append zeroes, if Nx < N
Nx1 = Nx-N;
ntime = 1 + floor(Nx1/M);           % number of time points of spectrogram
nfreq = N/2+1;
Xs = zeros(nfreq,ntime);            % matrix to hold spectra

% compute spectrogram

mcol = 0;
for m = 1:M:Nx1                     % (decimated) time loop
   mcol = mcol+1;
   xt = fft(x(m:m+N-1).*w,N);       % cut out and window signal block of length N, apply FFT
   Xs(:,mcol) = abs(xt(1:nfreq));   % take magnitude of spectrum
end

Xs = Xs/max(max(Xs));
if ~isempty(dB)
   Xs = 20*log10(Xs+eps);
   magmax = max(max(Xs));
   magmin = min(min(Xs));
   dB = abs(dB);
   if magmax - magmin > dB          % eliminate spectral nulls (set dynamic range to dB dB)
      Xs = Xs(:);
      imin = find(Xs < magmax-dB);
      Xs(imin) = (magmax-dB)*ones(length(imin),1);
      Xs = reshape(Xs,nfreq,ntime);
   end
end

if strcmp(upper(computer),'PCWIN')
   axfnt = 9;
   ticfnt = 8;
else	
   axfnt = 13;
   ticfnt = 12;
end

figurebackcolor = 'black';
fsig = figure('name','Spectrogram Display','Units','normal','Position',pos);
colordef(fsig,figurebackcolor);

nt = int2str(N);
mt = num2str(100*(N-M)/N);

f = linspace(0,0.5,nfreq);

if isempty(G_Fsampl)
   G_Fsampl = 16000;
   disp('Sampling frequency set to 16 kHz');
end
f = G_Fsampl*f/1000;
t1 = N/2 - M/2;
t2 = t1 + 1;
t = t2:M:Nx1+t2-1;
tmax = min(Nx+1,max(t));

subplot(2,1,1),imagesc(t/G_Fsampl,f,Xs), colormap(jet),
set(gca,'Xlim',[t2 tmax]/G_Fsampl);
set(gca,'YDir','normal');
ylabel('kHz');
hd = get(gca,'ylabel');
set(hd,'fontsize',axfnt);
if pos(4) > 0.38
   if ~isempty(dB)
      title(['LOG. SPECTROGRAM (Ham. ',nt,'  DFT ',nt,'  Overlap. ', mt,'%)']);
   else
      title(['SPECTROGRAM (Ham. ',nt,'  DFT ',nt,'  Overlap. ', mt,'%)']);
   end
end
set(gca,'fontsize',ticfnt);
hd = get(gca,'title');
set(hd,'fontsize',axfnt);

x = x(t1:Nx);
if ~exist('G_x_zoom') | isempty(G_x_zoom)
   G_x_zoom(:,gcf) =  x;
else
   [nr,nc] = size(G_x_zoom);
   Nxx = length(x);
   G_x_zoom = [[G_x_zoom ; zeros(Nxx-nr,nc)] [x;zeros(nr-Nxx,1)]];
end

subplot(2,1,2), plot([t2:Nx+1]/G_Fsampl,x), ylabel(['Signal  ',signame]),
set(gca,'Xlim',[t2 tmax]/G_Fsampl)
hd = get(gca,'ylabel');
set(hd,'fontsize',axfnt);
if pos(4) > 0.38
   xlabel('Seconds');
   hd = get(gca,'xlabel');
   set(hd,'fontsize',axfnt);
   bpos = [0.91,0.25,0.07,0.045];
else
   bpos = [0.91,0.22,0.07,0.090];
end
set(gca,'fontsize',ticfnt);

if pos(4) < 0.38
   text('String','Seconds','Units','normalized','fontsize',axfnt,'Position',[1.03 1.2]);
end
zoom(fsig,'xon');

uicontrol('Style','Pushbutton','Units','normalized',...
   'Position',bpos,'String','Play','Callback',...
   'specplay');
